/*
 * $Header: /cvsroot/jdbforms/dbforms/src/org/dbforms/event/eventtype/DatabaseEventTypeStrategy.java,v 1.6 2004/10/03 18:53:22 hkollmann Exp $
 * $Revision: 1.6 $
 * $Date: 2004/10/03 18:53:22 $
 *
 * DbForms - a Rapid Application Development Framework
 * Copyright (C) 2001 Joachim Peer <joepeer@excite.com>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 */

package org.dbforms.event.eventtype;


/**
 * Database EventTypeStrategy class. Provides implementations of the super
 * class methods for Database events.
 *
 * @author Luca Fossato
 *
 */
public class DatabaseEventTypeStrategy implements EventTypeStrategy {

   /**
    * Gets the eventGroup attribute of the EventTypeStrategyImpl object
    *
    * @param eventString Description of the Parameter
    *
    * @return The eventGroup value, or EventType.EVENT_UNDEFINED otherwise
    */
   public int getEventGroup(String eventString) {
      int eventGroup = EventType.EVENT_UNDEFINED;

      if ((eventString.startsWith("ac_insert_"))
                || (eventString.startsWith("ac_update_"))
                || (eventString.startsWith("ac_updatear_"))
                || (eventString.startsWith("ac_delete_"))
                || (eventString.startsWith("ac_deletear_"))) {
         eventGroup = EventType.EVENT_GROUP_DATABASE;
      }

      return eventGroup;
   }


   /**
    * Gets the eventType attribute of the EventTypeStrategyImpl object
    *
    * @param eventString Description of the Parameter
    *
    * @return The eventType value
    */
   public String getEventType(String eventString) {
      String eventType = String.valueOf(EventType.EVENT_UNDEFINED);

      if (eventString.startsWith("ac_insert_")) {
         eventType = EventType.EVENT_DATABASE_INSERT;
      } else if (eventString.startsWith("ac_update_")
                       || eventString.startsWith("ac_updatear_")) {
         eventType = EventType.EVENT_DATABASE_UPDATE;
      } else if (eventString.startsWith("ac_delete_")
                       || eventString.startsWith("ac_deletear_")) {
         eventType = EventType.EVENT_DATABASE_DELETE;
      }

      return eventType;
   }


   /**
    * Gets the EventTypeStrategy identifier.
    *
    * @return the EventTypeStrategy identifier
    */
   public String getId() {
      return "DatabaseEventTypeStrategy";
   }
}
